package projekt.enigma.model;

/**
 * Klasse Walze
 * <br>
 * Erzeugt ein Objekt des Typs Walze mit den Eigenschaften:
 * <br>
 * 1. Walzennummer (Die Walze enthaelt die 26 Buchstaben des Alphabets und codiert diese.
 * Fuenf verschiedene Walzen stehen zur Auswahl, die jeweils verschieden die Buchstaben des Alphabets
 * vertauschen).
 * <br>
 * 2. Ringstellung (Umspringpunkt der Walze wird festgelegt. Der Umspringpunkt bestimmt
 * den Drehzeitpunkt der linken Nachbarwalze).
 */
public class Walze {

	//region Variablen
	/**
	 * Die Anzahl der getaetigten Walzendrehungen.
	 */
	private int turns;

	/**
	 * Das String-Array aller bekannter aktueller Walzen.
	 */
	private String[] walzen;

	/**
	 * alphabet : String : enthaelt die Buchstaben des Alphabets.
	 */
	private String alphabet;

	/**
	 * ringstellung : char : zeigt die Einstellung fuer den Umspringpunkt.
	 */
	private char ringstellung;

	/**
	 * walzenNr : int : Bestimmt, welche Walze genutzt wird.
	 */
	private int walzenNr;

	//endregion

	//region Konstruktor
	/**
	 * Ueberschriebener Konstruktor, der die Eingangsparameter der Walze mit gibt.
	 *
	 * @param walzenNr     : int : Nummer der gewaehlten Walze.
	 * @param ringstellung : int : Einstellung des Umsprungpunktes.
	 */
	public Walze(int walzenNr, int ringstellung) {
		this.walzen = new String[5];
		this.walzen[0] = "EKMFLGDQVZNTOWYHXUSPAIBRCJ";
		this.walzen[1] = "AJDKSIRUXBLHWTMCQGZNPYFVOE";
		this.walzen[2] = "BDFHJLCPRTXVZNYEIWGAKMUSQO";
		this.walzen[3] = "ESOVPZJAYQUIRHXLNFTGKDCMWB";
		this.walzen[4] = "VZBRGITYUPSDNHLXAWMJQOFECK";
		this.alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		this.turns = 0;

		this.setWalzenNr(walzenNr);
		this.setRingstellung(ringstellung);
	}

	//endregion

	//region Funktionen & Methoden
	/**
	 * Dreht die Walze: Zaehlt die Umdrehungen (turns) um eins hoch oder runter, je nach Drehrichtung.
	 * <p>
	 * Ueberprueft, ob der Umspringpunkt der Walze nach der Drehung erreicht wurde.
	 *
	 * @param richtung : int : 1 dreht die Walze weiter und -1 dreht sie einen Schritt zurueck.
	 * @return checkRing : boolean : gibt true zurueck, wenn der Umspringpunkt erreicht wurde.
	 */
	boolean dreheWalze(int richtung) {
		boolean checkRing = false;
		int korrekturFaktor = 0;

		if ((richtung == -1) || (richtung == 1)) {
			if (richtung == 1) {
				this.turns++;
			} else {
				korrekturFaktor = 1;
				this.turns--;
			}

			// gleicht die Ringstellung mit der aktuellen Position ab
			if ((this.alphabet.indexOf(this.getPosition()) + korrekturFaktor) % 26
					== this.alphabet.indexOf(this.ringstellung)) {
				checkRing = true;
			}
		}

		return checkRing;
	}

	/**
	 * Codiert den mitgegebenen Buchstaben anhand der gewaehlten Walze.
	 *
	 * @param buchstabe : char : Buchstabe, der verschluesselt werden soll.
	 * @return char : verschluesselter Buchstabe.
	 */
	public char codiere(char buchstabe) {
		return this.fetchWalze().charAt((this.alphabet.indexOf(buchstabe) + this.turns) % 26);
	}

	/**
	 * Decodiert den mitgegebenen Buchstaben anhand des Alphabets.
	 *
	 * @param buchstabe : char : Buchstabe, der decodiert werden soll.
	 * @return char : decodierter Buchstabe.
	 */
	char codiere2(char buchstabe) {
		return this.alphabet.charAt((this.fetchWalze().indexOf(buchstabe) - this.turns + 260) % 26);
	}

	/**
	 * @return String : Gibt den Verschluesselungs-String der ausgewaehlten Walze zurueck.
	 */
	private String fetchWalze() {
		return walzen[walzenNr];
	}

	//endregion

	//region Setter
	/**
	 * Setzt die Walzennummer. Es stehen fuenf Walzen zur Auswahl.
	 *
	 * @param walzenNr : int : Walzennummer.
	 */
	private void setWalzenNr(int walzenNr) {
		if ((walzenNr > 0) && (walzenNr < 6)) {
			this.walzenNr = walzenNr - 1;
		} else {
			System.err.println("Keine gültige Walzennummer");
		}
	}

	/**
	 * Setzt die Initiale Ringstellung der Walze.
	 * Es sind nur Zahlen von 1 - 26 zugelassen.
	 * <br>
	 * Die Ringstellung wird zum char umgewandelt.
	 * Im Fehlerfall wird die Ringstellung standardmaessig auf 'Z' gesetzt.
	 *
	 * @param ringstellung : int : Punkt an dem die Nachbarwalze umspringt.
	 */
	void setRingstellung(int ringstellung) {
		if ((ringstellung > 0) && (ringstellung <= 26)) {
			this.ringstellung = this.alphabet.charAt(ringstellung - 1);
		} else {
			this.ringstellung = 'Z';
		}
	}

	/**
	 * Stellt die Grundeinstellung der Walze ein.
	 * Ermittelt, wie oft die Walze gedreht werden muss, bis die Grundeinstellung erreicht ist.
	 *
	 * @param buchstabe : Character : Einstellung der Walze.
	 */
	void setPosition(Character buchstabe) {
		this.turns = alphabet.indexOf(buchstabe);
	}
	//endregion

	//region Getter
	/**
	 * Gibt den Buchstaben zurueck, der aktuell in der Walze eingestellt ist.
	 *
	 * @return char : Ermittelte Stelle im Alphabet, anhand der aktuellen Anzahl der Drehungen.
	 */
	Character getPosition() {
		return this.alphabet.charAt(turns % 26);
	}

	/**
	 * Getter der Walzennummer.
	 * @return walzenNr : int : gibt Walzennummer zurueck.
	 */
	int getWalzenNr() {
		return this.walzenNr;
	}
	//endregion
}